"
stx:libscm - a new source code management library for Smalltalk/X
Copyright (C) 2012-2013 Jan Vrany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License. 

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
"
"{ Package: 'stx:libscm/common' }"

Object subclass:#SCMAbstractTask
	instanceVariableNames:'package classes temporaryWorkingCopy'
	classVariableNames:''
	poolDictionaries:''
	category:'SCM-Common-StX-Tasks'
!

!SCMAbstractTask class methodsFor:'documentation'!

copyright
"
stx:libscm - a new source code management library for Smalltalk/X
Copyright (C) 2012-2013 Jan Vrany

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License. 

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
"
! !

!SCMAbstractTask methodsFor:'accessing'!

branch

    ^self workingCopy branch

    "Created: / 23-03-2009 / 17:16:14 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

classes
    ^ classes
!

classes:aCollection"<Collection[Class]>"

    self 
        assert: (aCollection allSatisfy: [:e|e isBehavior])
        message: 'All elements should be classes'.

    self
        assert: (aCollection allSatisfy: [:e|e package = package name])
        message: 'All classes should belongs to my package (' , package name , ')'.

    classes := aCollection.

    "Modified: / 16-06-2009 / 20:56:53 / Jan Vrany <vranyj1@fel.cvut.cz>"
    "Modified: / 14-11-2012 / 00:38:35 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

package
    ^ package
!

package:aSCMAbstractPackageModel
    package := aSCMAbstractPackageModel.
    temporaryWorkingCopy := aSCMAbstractPackageModel temporaryWorkingCopy

    "Modified: / 11-01-2013 / 19:31:17 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

repository

    ^self workingCopy repository

    "Created: / 23-03-2009 / 11:24:36 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

temporaryWorkingCopy
    ^ temporaryWorkingCopy
!

temporaryWorkingCopy: wc
    temporaryWorkingCopy := wc

    "Created: / 12-01-2013 / 13:57:15 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

temporaryWorkingCopyRoot
    ^package notNil ifTrue:[
        package temporaryWorkingCopyRoot
    ] ifFalse:[
        temporaryWorkingCopy root
    ]

    "Created: / 14-11-2012 / 23:51:48 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 12-01-2013 / 13:55:30 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

workingCopy
    <resource: #obsolete>

    self breakPoint: #jv. 
    ^self temporaryWorkingCopy

    "Created: / 14-11-2012 / 23:51:38 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 12-01-2013 / 13:57:52 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

workingCopy: wc

    <resource: #obsolete>

    self breakPoint: #jv.
    self temporaryWorkingCopy: wc

    "Created: / 11-01-2013 / 19:34:48 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 12-01-2013 / 13:57:37 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

workingCopyRoot
   <resource: #obsolete>

    self breakPoint: #jv. 
    ^self temporaryWorkingCopyRoot

    "Created: / 11-01-2013 / 19:32:54 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 12-01-2013 / 13:56:32 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!SCMAbstractTask methodsFor:'executing'!

do
    "
        Perform whole task
    "            

    ^ self subclassResponsibility

    "Modified: / 23-03-2009 / 11:16:11 / Jan Vrany <vranyj1@fel.cvut.cz>"
! !

!SCMAbstractTask methodsFor:'executing - private'!

do: aBlock
    ^package notNil ifTrue:[
        SCMCompatModeQuery
            answer: package name
            do: aBlock
    ] ifFalse:[
        aBlock value.
    ]

    "Created: / 15-07-2009 / 20:07:14 / Jan Vrany <vranyj1@fel.cvut.cz>"
    "Modified: / 01-04-2013 / 12:19:57 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

doCompileSvnRepositoryUrlStringMethod
    |pkgDef revNr|

    ActivityNotification notify:'Compiling #svnRepositoryUrlString method'.
    pkgDef := self workingCopy packageDefinition.
    (pkgDef theMetaclass includesSelector:#svnRepositoryUrlString) not
            ifTrue:[
                pkgDef theMetaclass compile:(pkgDef svnRepositoryUrlString_code)
                    classified:'description - svn'.
                (pkgDef theMetaclass compiledMethodAt:#svnRepositoryUrlString) 
                    setPackage:self package asSymbol
            ].

    "Created: / 08-04-2011 / 15:58:14 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

doCompileSvnRevisionNrMethod:compileRevision 
    |pkgDef revNr|

    ActivityNotification notify:'Compiling #svnRevisionNr method'.
    pkgDef := self workingCopy packageDefinition.
    revNr := compileRevision ifTrue:[
                pkgDef svnRevision number
            ] ifFalse:[ nil ].
    (revNr notNil 
        or:[ (pkgDef theMetaclass includesSelector:#svnRevisionNr) not ]) 
            ifTrue:[
                pkgDef theMetaclass compile:(pkgDef svnRevisionNr_code:revNr)
                    classified:'description - svn'.
                (pkgDef theMetaclass compiledMethodAt:#svnRevisionNr) 
                    setPackage:self package asSymbol
            ].

    "Created: / 16-06-2009 / 12:16:25 / Jan Vrany <vranyj1@fel.cvut.cz>"
! !

!SCMAbstractTask methodsFor:'notification'!

notify: aString

    self notify: aString progress: nil

    "Created: / 29-05-2009 / 16:51:13 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

notify: aString progress: aNumberOrNil

    aNumberOrNil isNil ifTrue:[
        ActivityNotification notify: aString
    ] ifFalse:[
        ProgressNotification newException
                messageText:aString;
                parameter: aNumberOrNil;
                raiseRequest.
    ]

    "Created: / 29-05-2009 / 16:51:59 / Jan Vrany <vranyj1@fel.cvut.cz>"
    "Modified: / 30-06-2013 / 12:45:38 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!SCMAbstractTask class methodsFor:'documentation'!

version_GIT
    "Never, ever change this method. Ask JV or CG why"
    ^thisContext method mclass theNonMetaclass instVarNamed: #revision
!

version_HG

    ^ '$Changeset: <not expanded> $'
!

version_SVN
    ^ 'Id::                                                                                                                        '
! !

